#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

#from zsigen import AdminService_client as c
from ZSI import FaultException
from util import handle_faultexception, handle_exception
import sys

class AdminManagementPort:

    def __init__(self, urlbase, auth, transport, ovm_version):
        self.wspath = "/AdminServiceSoapHttpPort"
        global c       
        package = "zsigen." + ovm_version
        exec("from " + package + " import AdminService_client as client")
        c = client
        self.loc = c.AdminServiceLocator()
        self.srv = self.loc.getAdminServiceSoapHttpPort(url = urlbase + self.wspath, 
                                                        auth = auth,                                                        
                                                        transport = transport)
    def createUser(self, accountName, email, firstName, lastName, password, accountStatus, roleId):
        """ 
        Create a OVM manager user.     
        """
        request = c.AdminService_createUser4()
        request._accountName = accountName
        request._email       = email
        request._firstName   = firstName
        request._lastName    = lastName
        request._password    = password
        request._accountStatus = accountStatus
        request._roleId        = roleId     
        try:
            response = self.srv.createUser4(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def createGroup(self, groupName, desc):
        """ 
        Create a OVM manager user.     
        """
        request = c.AdminService_createGroup()
        request._groupName  = groupName
        request._desc       = desc   
        try:
            response = self.srv.createGroup(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def createUserGroup(self, userId, groupId):
        """ 
        return OVMUserGroup .     
        """
        request = c.AdminService_createUserGroup()
        request._userId = userId 
        request._groupId = groupId  
        try:
            response = self.srv.createUserGroup(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def createUserSite(self, userId, siteId):
        """ 
        return OVMUserSite .     
        """
        request = c.AdminService_createUserSite()
        request._userId = userId
        request._siteId = siteId  
        try:
            response = self.srv.createUserSite(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)             
            
    def findAllOVMUser(self):
        """ 
        return OVMUser[].     
        """
        request = c.AdminService_findAllOVMUser()  
        try:
            response = self.srv.findAllOVMUser(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def findAllOVMUserGroup(self):
        """ 
        return OVMUserGroup[].     
        """
        request = c.AdminService_findAllOVMUserGroup()  
        try:
            response = self.srv.findAllOVMUserGroup(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)                 

    def findAllOVMGroup(self):
        """ 
        return OVMGroup[].     
        """
        request = c.AdminService_findAllOVMGroup()  
        try:
            response = self.srv.findAllOVMGroup(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)   
 

    def findAllOVMRole(self):
        """ 
        return OVMRole[].     
        """
        request = c.AdminService_findAllOVMRole()  
        try:
            response = self.srv.findAllOVMRole(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)  
            
    def findAllOVMUserRole(self):
        """ 
        return OVMUserRole[].     
        """
        request = c.AdminService_findAllOVMUserRole()  
        try:
            response = self.srv.findAllOVMUserRole(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)     

    def findAllOVMUserSite(self):
        """ 
        return OVMUserSite[].     
        """
        request = c.AdminService_findAllOVMUserSite()  
        try:
            response = self.srv.findAllOVMUserSite(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def findUserById(self):
        """ 
        return OVMUser[].     
        """
        request = c.AdminService_findUserById()  
        try:
            response = self.srv.findUserById(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def login(self, accountName, password, extProps):
        """ 
        return string.     
        """
        request = c.AdminService_login()
        request._accountName = accountName
        request._password = password
        request._extProps = extProps 
        try:
            response = self.srv.login(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def logout(self, token, accountName, password):
        """ 
        return boolean.     
        """
        request = c.AdminService_logout()
        request._accountName = accountName
        request._password = password
        request._token = token 
        try:
            response = self.srv.logout(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def get_user_group_names(self, user_id):
        result = []
        user_groups = self.findAllOVMUserGroup()    
        for user_group in user_groups:
            if user_group._group_id == 0 or user_group._group_id == -1:
                continue
            if user_group._user_id == user_id and user_group._status == 'in':
                group_name = self.get_group_by_id(user_group._group_id)
                result.append(group_name)
        return result
    
    def get_group_by_id(self, group_id):
        groups = self.findAllOVMGroup()
        for group in groups:
            if group._groupId == group_id:
                return group._groupName

    def get_groupid_by_name(self, group_name):
        groups = self.findAllOVMGroup()
        for group in groups:
            if group._groupName == group_name:
                return group._groupId
        return None

    def get_group_by_name(self, group_name):
        groups = self.findAllOVMGroup()
        for group in groups:
            if group._groupName == group_name:
                return group
        return None

    def get_group_user_number(self, group_id):
        count = 0
        user_groups = self.findAllOVMUserGroup()    
        for user_group in user_groups:
            if user_group._group_id == group_id:
                count += 1
        return count
                    
    def get_user_role_names(self, user_id):
        result = []
        user_roles = self.findAllOVMUserRole()    
        for user_role in user_roles:
            if user_role._user_id == user_id:
                role_name = self.get_role_by_id(user_role._role_id)
                result.append(role_name)
        return result
    
    def get_role_by_id(self, role_id):
        roles = self.findAllOVMRole()
        for role in roles:
            if role._role_id == role_id:
                return role._role_name

    def get_roleid_by_name(self, role_name):
        roles = self.findAllOVMRole()
        for role in roles:
            if role._role_name == role_name:
                return role._role_id
        return None

    def get_all_role_names(self):
        result = []
        roles = self.findAllOVMRole()    
        for role in roles:
            result.append(role._role_name)       
        return result

    def get_all_group_names(self):
        result = []
        groups = self.findAllOVMGroup()    
        for group in groups:
                result.append(group._groupName)
        return result                                        

    def get_userid_by_name(self, user_name):
        users = self.findAllOVMUser()
        for user in users:
            if user._accountName == user_name:
                return user._userId
        return None

    def get_user_site_ids(self, user_id):
        result = []
        user_sites = self.findAllOVMUserSite()    
        for user_site in user_sites:
            if user_site._user_id == user_id:
                result.append(user_site._site_id)
        return result
    
    def get_all_admin_ids(self):
        result = []
        users = self.findAllOVMUserRole()
        for user in users:
            if user._role_id == 1:
                result.append(user._user_id)
        return result

    def get_userids_by_names(self, user_names):
        result = []
        if user_names == []:
            return []
        users = self.findAllOVMUser()
        for user_name in user_names:
            exist = False
            for user in users:
                if user._accountName == user_name:
                    result.append(user._userId)
                    exist = True
                    break
            if not exist:
                return user_name
        return result

    def get_userids_by_siteid(self, serverPoolId):
        result = []
        usersites = self.findAllOVMUserSite()
        for usersite in usersites:
            if usersite._site_id == serverPoolId:
                result.append(usersite._user_id)
        return result        