#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

#from zsigen import LifecycleService_client as c
from ZSI import FaultException
from util import handle_faultexception, handle_exception
import sys

class LifecycleManagementPort:

    def __init__(self, urlbase, auth, transport, ovm_version):          
        self.wspath = "/LifecycleServiceSoapHttpPort"
        global c
        package = "zsigen." + ovm_version
        exec("from " + package + " import LifecycleService_client as client")
        c = client
        self.loc = c.LifecycleServiceLocator()
        self.srv = self.loc.getLifecycleServiceSoapHttpPort(url = urlbase + self.wspath, 
                                                            auth = auth,                                                                                                                         #tracefile=sys.stdout,
                                                            transport = transport)

    def addDisk(self, virtualMachine, diskConfig):
        """ 
        Create a new disk image on the server pool, and attaches it to the virtual machine.
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param diskConfig: 
        @type diskConfig: DiskConfig
        @return: VirtualDisk       
        """
        request = c.LifecycleService_addDisk()
        request._virtualMachine       = virtualMachine
        request._diskConfig           = diskConfig
        try:
            response = self.srv.addDisk(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 
    
    def addNIC(self, virtualMachine, nicConfig):
        """ 
        Add a new network interface card to the virtual machine. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param nicConfig: 
        @type nicConfig: NICConfig
        @return:  VirtualNetworkCard           
        """
        request = c.LifecycleService_addNIC()
        request._virtualMachine       = virtualMachine
        request._nicConfig            = nicConfig
        try:
            response = self.srv.addNIC(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def attachSharedVirtualDisk(self, virtualMachine, sharedDisk):
        """ 
        @return:  void 
        """
        request = c.LifecycleService_attachSharedVirtualDisk()
        request._virtualMachine       = virtualMachine
        request._sharedDisk           = sharedDisk
        try:
            response = self.srv.attachSharedVirtualDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def attachCDtoVM(self, virtualMachine, cdConfig): 
        """" 
        Attach a CD to the virtual machine.
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param cdConfig: 
        @type cdConfig: CdConfig
        @return:  void 
        """ 
        request = c.LifecycleService_attachCDtoVM()
        request._virtualMachine           = virtualMachine
        request._cdConfig                 = cdConfig
        try:
            response = self.srv.attachCDtoVM(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def attachDisk(self, disk): 
        """ 
        Attach a disk (previously detached) to the virtual machine.
        @param disk: 
        @type disk: VirtualDisk
        @return:  void 
        """ 
        request = c.LifecycleService_attachDisk()
        request._disk       = disk
        try:
            response = self.srv.attachDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def attachNIC(self, nic): 
        """ 
        Attach a network interface card to a virtual machine. 
        @param nic: 
        @type nic: VirtualNetworkCard
        @return:  void 
        """ 
        request = c.LifecycleService_attachNIC()
        request._nic       = nic
        try:
            response = self.srv.attachNIC(request)
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def clone(self, conf): 
        """ 
        Clone a virtual machine. 
        @param conf: 
        @type conf: CloneVmConfig
        @return:  VirtualMachine 
        """ 
        request = c.LifecycleService_clone()
        request._conf       = conf
        try:
            response = self.srv.clone(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def deploy(self, conf): 
        """ 
        Deploy a virtual machine. 
        @param conf: 
        @type conf: DeployVmConfig
        @return:  VirtualMachine 
        """ 
        request = c.LifecycleService_deploy()
        request._conf       = conf
        try:
            response = self.srv.deploy(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def configDisk(self, imgId, virtualDisk): 
        """ 
        @return:  void 
        """ 
        request = c.LifecycleService_configDisk()
        request._imgId             = imgId
        request._virtualDisk       = virtualDisk
        try:
            response = self.srv.configDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def configNIC(self, imgId, nic): 
        """ 
        @return:  void 
        """ 
        request = c.LifecycleService_configNIC()
        request._imgId     = imgId
        request._nic       = nic
        try:
            response = self.srv.configNIC(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def configVMGroup(self, imgId, groupId): 
        """ 
        @return:  void 
        """ 
        request = c.LifecycleService_configVMGroup()
        request._imgId     = imgId
        request._groupId       = groupId
        try:
            response = self.srv.configVMGroup(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def setVCPUCapByVMID(self, vmId, cap): 
        """ 
        @return:  void 
        """ 
        request = c.LifecycleService_setVCPUCapByVMID()
        request._vmId     = vmId
        request._cap       = cap
        try:
            response = self.srv.setVCPUCapByVMID(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)


    def setVCPUPriorityByVMID(self, vmId, priority): 
        """ 
        @return:  void 
        """ 
        request = c.LifecycleService_setVCPUPriorityByVMID()
        request._vmId     = vmId
        request._priority       = priority
        try:
            response = self.srv.setVCPUPriorityByVMID(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)                                    
                
    def createVmBasedOnISO(self, conf): 
        """ 
        Create a new virtual machine, using an available ISO Images resource on the server pool.
        @param conf: 
        @type conf: ISOVMConfig 
        @return:  VirtualMachine 
        """ 
        # TOFIX
        request = c.LifecycleService_createVmBasedOnISO()
        request._conf       = conf
        try:
            response = self.srv.createVmBasedOnISO(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def createVmBasedOnPXE(self, pxeVMConfig): 
        """ 
        Create a new virtual machine (configured to use PXE boot mechanism) on the server pool. 
        @param pxeVMConfig: 
        @type pxeVMConfig: PXEVMConfig
        @return:  VirtualMachine      
        """ 
        # TOFIX
        request = c.LifecycleService_createVmBasedOnPXE()
        request._pxeVMConfig       = pxeVMConfig
        try:
            response = self.srv.createVmBasedOnPXE(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def createVmBasedOnTemplate(self, templateVMConfig): 
        """ 
        Create a new virtual machine, using an available Oracle VM Template resource on the server pool. 
        @param templateVMConfig: 
        @type templateVMConfig: TemplateVMConfig 
        @return:  VirtualMachine         
        """ 
        # TOFIX
        request = c.LifecycleService_createVmBasedOnTemplate()
        request._templateVMConfig       = templateVMConfig
        try:
            response = self.srv.createVmBasedOnTemplate(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def detachCD(self, virtualMachine): 
        """
        Detach a CD from the virtual machine. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @return:  void 
        """ 
        request = c.LifecycleService_detachCD()
        request._virtualMachine       = virtualMachine
        try:
            response = self.srv.detachCD(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def detachDisk(self, disk): 
        """ 
        Detach a disk from the virtual machine, preserving it physically.
        @param disk: 
        @type disk: VirtualDisk
        @return:  void 
        """ 
        request = c.LifecycleService_detachDisk()
        request._disk       = disk
        try:
            response = self.srv.detachDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def detachNIC(self, nic): 
        """ 
        Detach a network interface card from a virtual machine. 
        @param nic: 
        @type nic: VirtualNetworkCard 
        @return:  void         
        """ 
        request = c.LifecycleService_detachNIC()
        request._nic       = nic
        try:
            response = self.srv.detachNIC(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def disableHA(self, vm): 
        """ 
        Disable virtual machine high availability. 
        @param vm: 
        @type vm: VirtualMachine
        @return:  void         
        """ 
        request = c.LifecycleService_disableHA()
        request._vm       = vm
        try:
            response = self.srv.disableHA(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def enableHA(self, vm): 
        """ 
        Enable virtual machine high availability. 
        @param vm: 
        @type vm: VirtualMachine
        @return:  void         
        """ 
        request = c.LifecycleService_enableHA()
        request._vm       = vm
        try:
            response = self.srv.enableHA(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def liveMigrate(self, virtualMachine): 
        """ 
        Live Migrates, a virtual machines to another virtual server (on the server pool). 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @return:  void         
        """ 
        request = c.LifecycleService_liveMigrate()
        request._virtualMachine       = virtualMachine
        try:
            response = self.srv.liveMigrate(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def liveMigrateToServer(self, virtualMachine, virtualServer): 
        """ 
        Live Migrates, a virtual machines to a specified server. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param virtualServer: 
        @type virtualServer: VirtualServer 
        @return:  void     
        """ 
        # TOFIX
        request = c.LifecycleService_liveMigrate2()
        request._virtualMachine       = virtualMachine
        request._virtualServer        = virtualServer
        try:
            response = self.srv.liveMigrate2(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def liveMigrateAll(self, virtualServer): 
        """ 
        Live Migrates all virtual machines being hosted on a virtual server, to other virtual servers (on the server pool). 
        @param virtualServer: 
        @type virtualServer: VirtualServer 
        @return:  void 
        """ 
        request = c.LifecycleService_liveMigrateAll()
        request._virtualServer        = virtualServer
        try:
            response = self.srv.liveMigrateAll(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def liveMigrateAllToServer(self, srcServer, tgtServer): 
        """ 
        @return:  void 
        """ 
        request = c.LifecycleService_liveMigrateAll2()
        request._srcServer        = srcServer
        request._tgtServer        = tgtServer
        try:
            response = self.srv.liveMigrateAll2(request)
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)        
    def removeDisk(self, disk): 
        """ 
        Detach a disk from the virtual machine, and removes it physically from the server pool. 
        @param disk: 
        @type disk: VirtualDisk
        @return:  void 
        """ 
        request = c.LifecycleService_removeDisk()
        request._disk        = disk
        try:
            response = self.srv.removeDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def removeNIC(self, nic): 
        """ 
        Remove an existing network interface card from the virtual machine. 
        @param nic: 
        @type nic: VirtualNetworkCard
        @return:  void         
        """ 
        request = c.LifecycleService_removeNIC()
        request._nic        = nic
        try:
            response = self.srv.removeNIC(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def detachSharedVirtualDisk(self, virtualMachine, sharedDisk): 
        """ 
        Detach a Shared Disk resource from a virtual machine.
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param sharedDisk: 
        @type sharedDisk: SharedVirtualDisk 
        @return:  void 
        """ 
        request = c.LifecycleService_detachSharedVirtualDisk()
        request._virtualMachine        = virtualMachine
        request._sharedDisk            = sharedDisk
        try:
            response = self.srv.detachSharedVirtualDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def setBootPriority(self, virtualMachine, val): 
        """ 
        Set the boot priority of the virtual machine (takes affect on next boot).
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param val: 
        @type val: VirtualMachineBase.BootDevice 
        @return:  void         
        """ 
        request = c.LifecycleService_setBootPriority()
        request._virtualMachine        = virtualMachine
        request._val                   = val
        try:
            response = self.srv.setBootPriority(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def setCpuCores(self, virtualMachine, val): 
        """ 
        Set the current number of virtual cpu cores available to the virtual machine. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param val: 
        @type val: Integer
        @return:  void 
        """ 
        request = c.LifecycleService_setCpuCores()
        request._virtualMachine        = virtualMachine
        request._val                   = val
        try:
            response = self.srv.setCpuCores(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def setDynamicMemory(self, virtualMachine, val): 
        """ 
        Set the current memory value allocated to the virtual machine. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param val: 
        @type val: Integer
        @return:  void         
        """ 
        request = c.LifecycleService_setDynamicMemory()
        request._virtualMachine        = virtualMachine
        request._val                   = val
        try:
            response = self.srv.setDynamicMemory(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def setMaximumMemory(self, virtualMachine, val): 
        """ 
        Set the maximum memory value, that the virtual machine can be allocated.
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param val: 
        @type val: Integer
        @return:  void   
        """ 
        request = c.LifecycleService_setMaximumMemory()
        request._virtualMachine        = virtualMachine
        request._val                   = val
        try:
            response = self.srv.setMaximumMemory(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def setOperatingSystem(self, vm, os): 
        """ 
        Set operating system. 
        @param vm: 
        @type vm: VirtualMachine
        @param os: 
        @type os: OperatingSystem
        @return:  void 
        """ 
        request = c.LifecycleService_setOperatingSystem()
        request._vm        = vm
        request._os        = os
        try:
            response = self.srv.setOperatingSystem(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
        
    def setPreferredServers(self, virtualMachine, preferredServers): 
        """ 
        Set preferred servers. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param preferredServers: 
        @type preferredServers: VirtualServer[]
        @return:  void         
        """ 
        request = c.LifecycleService_setPreferredServers()
        request._virtualMachine          = virtualMachine
        request._preferredServers        = preferredServers
        try:
            response = self.srv.setPreferredServers(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def setVMConfigParam(self, vmId, paramKey, paramValue): 
        """ 
        Set parameter's key and value for vm.cfg 
        @param vmId: 
        @type vmId: long
        @param paramKey: 
        @type paramKey: str
        @param paramValue: 
        @type paramValue: str
        @return:  void         
        """ 
        request = c.LifecycleService_setVMConfigParam()
        request._vmId         = vmId
        request._paramKey          = paramKey
        request._paramValue        = paramValue
        try:
            response = self.srv.setVMConfigParam(request)
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def setVMDescription(self, virtualMachine, val): 
        """ 
        Sets the virtual machine description. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param val: 
        @type val: str
        @return:  void         
        """ 
        request = c.LifecycleService_setVMDescription()
        request._virtualMachine          = virtualMachine
        request._val                     = val
        try:
            response = self.srv.setVMDescription(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    
    def resetStatus(self, virtualMachine): 
        """ 
        @return:  void         
        """ 
        request = c.LifecycleService_resetStatus()
        request._virtualMachine          = virtualMachine
        try:
            response = self.srv.resetStatus(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)        
 

    def setConsolePassword(self, vm, password): 
        """ 
        @return:  void 
        """ 
        request = c.LifecycleService_setConsolePassword()
        request._vm        = vm
        request._password        = password
        try:
            response = self.srv.setConsolePassword(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def setVMKeyboardLayout(self, vmId, keyboardLayout): 
        """ 
        @return:  void 
        """ 
        request = c.LifecycleService_setVMKeyboardLayout()
        request._vmId        = vmId
        request._keyboardLayout        = keyboardLayout
        try:
            response = self.srv.setVMKeyboardLayout(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def getKeyboardLayoutArray(self): 
        """ 
        @return:  KeyboardLayout[]
        """ 
        request = c.LifecycleService_getKeyboardLayoutArray()
        try:
            response = self.srv.getKeyboardLayoutArray(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            

    def getVMGroupByVMId(self, vmId): 
        """ 
        @return:  OVMGroup
        """ 
        request = c.LifecycleService_getVMGroupByVMId()
        request._vmId = vmId
        try:
            response = self.srv.getVMGroupByVMId(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def changeNetworkType(self, vm, type): 
        """ 
        @return:  void
        """ 
        request = c.LifecycleService_changeNetworkType()
        request._vm   = vm
        request._type = type
        try:
            response = self.srv.changeNetworkType(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)           
   
