"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

import sys, os
from optparse import Option, OptionParser, OptParseError
import gettext

_ = gettext.gettext

REQUIRED = 'required'
CHECK  = 'check'

class OvmOption (Option):
    ATTRS = Option.ATTRS + [REQUIRED] + [CHECK]
    
    def __init__(self, *opts, **attrs):
        if attrs.get(REQUIRED, False):
            attrs['help'] = '(Required) ' + attrs.get('help', "")
        Option.__init__(self, *opts, **attrs)
        
    def take_action (self, action, dest, opt, value, values, parser):
        if action == "help":
            raise UtilOptionError, parser.format_help()
        else:
            Option.take_action(
                self, action, dest, opt, value, values, parser)

class OvmOptionParser (OptionParser):
    def __init__(self, *args, **kw):
        kw['option_class'] = OvmOption
        apply(OptionParser.__init__, (self,) + args, kw)
        self._error_msg = None    

    def _add_help_option(self):
        self.add_option("-h", "--help",
                        action="help",
                        help=_("Display this help message"))
        
    def parse_args(self, args=None, values=None):
        self._error_msg = None
        try:
            options, args = OptionParser.parse_args(self, args, values)
        except (UtilOptionError, UnknownOptionError, ValueError), msg:
            self._error_msg = msg
            options, args = None, ()
        return options, args
    
    def check_values(self, values, args):
        for option in self.option_list:
            if hasattr(option, REQUIRED) \
                and option.required  \
                and hasattr(option, CHECK) \
                and option.check is None: 
                if not getattr(values, option.dest):
                    self.error("option %s is required." % (str(option)))
        return OptionParser.check_values(self, values, args)

    def error(self, msg):
        raise UnknownOptionError, msg
    
    def is_error(self):
        if self._error_msg is not None:
            return True

    def get_error_msg(self):
        return self._error_msg
    
    def _get_encoding(self, file):
        encoding = getattr(file, "encoding", None)
        if not encoding:
            encoding = sys.getdefaultencoding()
        return encoding
    
    def print_help(self, file=None):
        if file is None:
            file = sys.stdout
        encoding = self._get_encoding(file)
        file.write(unicode(self.format_help(), "UTF-8").encode(encoding, "replace"))
        
    def get_usage(self):
        self.usage = self.usage.replace('_', ' ')
        return OptionParser.get_usage(self)

            
class UnknownOptionError(OptParseError):
    pass

class UtilOptionError(OptParseError):
    pass