#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

#from zsigen import ResourceService_client as c
from ZSI import FaultException
from util import handle_faultexception, handle_exception
import sys
class ResourceManagementPort:

    def __init__(self, urlbase, auth, transport, ovm_version):
        self.wspath = "/ResourceServiceSoapHttpPort"
        global c
        package = "zsigen." + ovm_version
        exec("from " + package + " import ResourceService_client as client")
        c = client
        self.loc = c.ResourceServiceLocator()
        self.srv = self.loc.getResourceServiceSoapHttpPort(url = urlbase + self.wspath, 
                                                           auth = auth,                                                            
                                                           transport = transport)
         
    def approveTemplate(self, serverPoolName, templateName):
        """
        @return:  VirtualMachineTemplate 
        """
        request = c.ResourceService_approveTemplate()
        request._serverPoolName = serverPoolName
        request._templateName   = templateName
        try:
            response = self.srv.approveTemplate(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def approveImage(self, serverPoolName, imageName):
        """
        @return:  VirtualMachineImage 
        """
        request = c.ResourceService_approveImage()
        request._serverPoolName = serverPoolName
        request._imageName      = imageName
        try:
            response = self.srv.approveImage(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def approveISOResource(self, isoResourceId): 
        """ 
        Deletes the ISO from server pool (physically also) 
        @param isoResourceID: 
        @type isoResourceID: Long
        @return:  ISOResource 
        """ 
        request = c.ResourceService_approveISOResource()
        request._isoResourceId = isoResourceId
        try:
            response = self.srv.approveISOResource(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                                          
    def deleteISO(self, isoResourceId): 
        """ 
        Deletes the ISO from server pool (physically also) 
        @param isoResourceID: 
        @type isoResourceID: Long
        @return:  void  
        """ 
        request = c.ResourceService_deleteISO()
        request._isoResourceId = isoResourceId
        try:
            response = self.srv.deleteISO(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
      
      
    def deleteTemplate(self, serverPoolName, templateName): 
        """ 
        Deletes the template from server pool (physically also) 
        @return:  void          
        """ 
        request = c.ResourceService_deleteTemplate()
        request._serverPoolName = serverPoolName
        request._templateName   = templateName
        try:
            response = self.srv.deleteTemplate(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def deleteImage(self, serverPoolName, imageName): 
        """ 
        Deletes the template from server pool (physically also) 
        @return:  void          
        """ 
        request = c.ResourceService_deleteImage()
        request._serverPoolName = serverPoolName
        request._imageName      = imageName
        try:
            response = self.srv.deleteImage(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
      
    def getAllDiscoverableISO(self, serverPoolName): 
        """
        @return: DiscoveredISO[]         
        """
        request = c.ResourceService_getAllDiscoverableISO()
        request._serverPoolName =serverPoolName
        try:
            response = self.srv.getAllDiscoverableISO(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
      
    def getAllISOResources(self, serverPoolName): 
        """ 
        Returns an array of all ISO Image resource instances (and physically available) 
        registered with the server pool. 
        @return: ISOResource[]          
        """ 
        request = c.ResourceService_getAllISOResources()
        request._serverPoolName = serverPoolName
        try:
            response = self.srv.getAllISOResources(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getAllDiscoverableTemplates(self, serverPoolName): 
        """  
        @return: DiscoveredVirtualMachineTemplate[]         
        """ 
        request = c.ResourceService_getAllDiscoverableTemplates()
        request._serverPoolName = serverPoolName
        try:
            response = self.srv.getAllDiscoverableTemplates(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def getAllImageResources(self, serverPoolName): 
        """ 
        @return: VirtualMachineImage[]          
        """ 
        request = c.ResourceService_getAllImageResources()
        request._serverPoolName = serverPoolName
        try:
            response = self.srv.getAllImageResources(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getAllTemplateResources(self, serverPoolName): 
        """ 
        @return: VirtualMachineTemplate[]          
        """ 
        request = c.ResourceService_getAllTemplateResources()
        request._serverPoolName = serverPoolName
        try:
            response = self.srv.getAllTemplateResources(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getTemplateResourceByName(self, serverPoolName, templateName, silent = True):
        """ 
        @return:  VirtualMachineTemplate           
        """
        request = c.ResourceService_getTemplateResourceByName()
        request._serverPoolName = serverPoolName
        request._templateName = templateName
        try:
            response = self.srv.getTemplateResourceByName(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)
            
    def registerExternalImage(self, serverPoolName, virtualMachineImage): 
        """ 
        @return: void          
        """ 
        request = c.ResourceService_registerExternalImage()
        request._serverPoolName = serverPoolName
        request._virtualMachineImage = virtualMachineImage
        try:
            response = self.srv.registerExternalImage(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 
                                             
    def registerInternalImage(self, serverPoolName, virtualMachineImage): 
        """ 
        @return: void          
        """ 
        request = c.ResourceService_registerInternalImage()
        request._serverPoolName = serverPoolName
        request._virtualMachineImage = virtualMachineImage
        try:
            response = self.srv.registerInternalImage(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)                   

    def registerExternalTemplate(self, serverPoolName, virtualMachineTemplate): 
        """ 
        @return: void          
        """ 
        request = c.ResourceService_registerExternalTemplate()
        request._serverPoolName = serverPoolName
        request._virtualMachineTemplate = virtualMachineTemplate
        try:
            response = self.srv.registerExternalTemplate(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def registerInternalTemplate(self, serverPoolName, virtualMachineTemplate): 
        """ 
        @return: void          
        """ 
        #a = c.ResourceService_registerInternalTemplate
        request = c.ResourceService_registerInternalTemplate()
        request._serverPoolName = serverPoolName
        request._virtualMachineTemplate = virtualMachineTemplate
        try:
            response = self.srv.registerInternalTemplate(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def registerInternalISO(self, serverPoolName, isoResourceConfig): 
        """ 
        @return: void          
        """ 
        request = c.ResourceService_registerInternalISO()
        request._serverPoolName = serverPoolName
        request._isoResourceConfig = isoResourceConfig
        try:
            response = self.srv.registerInternalISO(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def deleteSharedVirtualDisk(self, sharedVirtualDiskId): 
        """ 
        @return: void          
        """ 
        request = c.ResourceService_deleteSharedVirtualDisk()
        request._sharedVirtualDiskId = sharedVirtualDiskId
        try:
            response = self.srv.deleteSharedVirtualDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getAllDiscoverableImages(self, serverPoolName): 
        """ 
        @return: DiscoveredVirtualMachineImage[]         
        """ 
        request = c.ResourceService_getAllDiscoverableImages()
        request._serverPoolName = serverPoolName
        try:
            response = self.srv.getAllDiscoverableImages(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getAllSharedVirtualDisks(self, serverPoolName): 
        """ 
        @return: SharedVirtualDisk[]         
        """ 
        request = c.ResourceService_getAllSharedVirtualDisks()
        request._serverPoolName = serverPoolName
        try:
            response = self.srv.getAllSharedVirtualDisks(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getImageResourceByName(self, serverPoolName, imageName, silent=True): 
        """ 
        @return: VirtualMachineImage         
        """ 
        request = c.ResourceService_getImageResourceByName()
        request._serverPoolName = serverPoolName
        request._imageName = imageName
        try:
            response = self.srv.getImageResourceByName(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)

            
    def getSharedDiskResourceByName(self, serverPoolName, sharedVirtualDiskName, silent=True): 
        """ 
        @return: SharedVirtualDisk         
        """ 
        request = c.ResourceService_getSharedDiskResourceByName()
        request._serverPoolName = serverPoolName
        request._sharedVirtualDiskName = sharedVirtualDiskName
        try:
            response = self.srv.getSharedDiskResourceByName(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)

    def registerExternalISO(self, serverPoolName, isoResourceConfig): 
        """ 
        @return: void         
        """ 
        request = c.ResourceService_registerExternalISO()
        request._serverPoolName = serverPoolName
        request._isoResourceConfig = isoResourceConfig
        try:
            response = self.srv.registerExternalISO(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def registerSharedVirtualDisk(self, serverPoolName, sharedVirtualDiskConfig): 
        """ 
        @return: void         
        """ 
        request = c.ResourceService_registerSharedVirtualDisk()
        request._serverPoolName = serverPoolName
        request._sharedVirtualDiskConfig = sharedVirtualDiskConfig
        try:
            response = self.srv.registerSharedVirtualDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getISOResourceByGroupNameAndISOName(self, serverPoolName, groupName, isoName): 
        """ 
        @return: ISOResource         
        """ 
        request = c.ResourceService_getISOResourceByGroupNameAndISOName()
        request._serverPoolName = serverPoolName
        request._groupName = groupName
        request._isoName   = isoName
        try:
            response = self.srv.getISOResourceByGroupNameAndISOName(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getUnregisteredMultipathDevices(self, serverPoolName): 
        """ 
        @return: DiscoveredSharedVirtualDisk[]       
        """ 
        request = c.ResourceService_getUnregisteredMultipathDevices()
        request._serverPoolName = serverPoolName
        try:
            response = self.srv.getUnregisteredMultipathDevices(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getUnregisteredSharedVirtualDisks(self, serverPoolName): 
        """ 
        @return: DiscoveredSharedVirtualDisk[]       
        """ 
        request = c.ResourceService_getUnregisteredSharedVirtualDisks()
        request._serverPoolName = serverPoolName
        try:
            response = self.srv.getUnregisteredSharedVirtualDisks(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def registerMultipathDevice(self, serverPoolName, sharedVirtualDiskConfig): 
        """ 
        @return: void    
        """ 
        request = c.ResourceService_registerMultipathDevice()
        request._serverPoolName = serverPoolName
        request._sharedVirtualDiskConfig = sharedVirtualDiskConfig
        try:
            response = self.srv.registerMultipathDevice(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 

    def registerInternalSharedVirtualDisk(self, serverPoolName, sharedVirtualDiskConfig): 
        """ 
        @return: void    
        """ 
        request = c.ResourceService_registerInternalSharedVirtualDisk()
        request._serverPoolName = serverPoolName
        request._sharedVirtualDiskConfig = sharedVirtualDiskConfig
        try:
            response = self.srv.registerInternalSharedVirtualDisk(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)            
                       
