#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

#from zsigen import ServerPoolService_client as c
from ZSI import FaultException
import sys
import constants as CONSTANTS
from util import handle_faultexception, handle_exception


class ServerPoolManagementPort:

    def __init__(self, urlbase, auth, transport, ovm_version):
        self.wspath = "/ServerPoolServiceSoapHttpPort"
        global c
        package = "zsigen." + ovm_version
        exec("from " + package + " import ServerPoolService_client as client")     
        c = client
            
        self.loc = c.ServerPoolServiceLocator()
        self.srv = self.loc.getServerPoolServiceSoapHttpPort(url = urlbase + self.wspath, 
                                                             auth = auth, 
                                                             #tracefile=sys.stdout, 
                                                             transport = transport)
        
    def addMembers(self, serverPool, serverConfigs): 
        """ 
        Add an array of virtual servers, to a server pool.
        @param serverPool: 
        @type serverPool: ServerPool
        @param serverConfigs: 
        @type serverConfigs: VirtualServerConfig[]     
        @return VirtualServer[]
        """
        request = c.ServerPoolService_addMembers()
        request._serverPool          = serverPool
        request._serverConfigs       = serverConfigs
        try:
            response = self.srv.addMembers(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def blackoutServer(self, virtualServerID): 
        """     
        @return void
        """
        request = c.ServerPoolService_blackoutServer()
        request._virtualServerID          = virtualServerID
        try:
            response = self.srv.blackoutServer(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
  
    def checkVirtualServerCompatibility(self, masterVS, vs): 
        """ 
        Check Virtual server compatibility. 
        1.Check CPU Architecture; 
        2.Check CPU Platform.
        @param masterVS:
        @type masterVS: MasterVirtualServer
        @param vs:
        @type vs: VirtualServer
        @return: void
        """
        request = c.ServerPoolService_checkVirtualServerCompatibility()
        request._masterVS          = masterVS
        request._vs                = vs
        try:
            response = self.srv.checkVirtualServerCompatibility(request)
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                
    def createServerPool(self, serverPoolConfig, userIDList): 
        """ 
        Create a new server pool for the system.  
        @param serverPoolConfig:
        @type serverPoolConfig: ServerPoolConfig
        @param userIDList:
        @type userIDList: array
        @return: ServerPool  
        """
        request = c.ServerPoolService_createServerPool()
        request._serverPoolConfig = serverPoolConfig     
        request._userIDList  = userIDList
        try:
            response = self.srv.createServerPool(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
             
    def deleteServerPool(self, serverPool, isDelWorkingDir = False, force = False, warningMessageList = None): 
        """ 
        Remove a server pool completely from the system .  
        @param serverPool: 
        @type serverPool: ServerPool
        @param isDelWorkDirectory:
        @type isDelWorkDirectory: boolean 
        @param force:  
        @type force: boolean 
        @param warningMessageList:  
        @type warningMessageList: list 
        @return: void
        """
        
        c.ServerPoolService_deleteServerPool()
        request = c.ServerPoolService_deleteServerPool()
        request._serverPool               = serverPool
        request._isDelWorkingDir          = isDelWorkingDir
        request._force                    = force
        try:
            response = self.srv.deleteServerPool(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def endBlackoutServer(self, virtualServerID): 
        """     
        @return void
        """
        request = c.ServerPoolService_endBlackoutServer()
        request._virtualServerID          = virtualServerID
        try:
            response = self.srv.endBlackoutServer(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def getAgentVersion(self, serverId): 
        """ 
        Return all virtual servers for the server pool.  
        @param serverId: 
        @type serverId: Long
        @return: str
        """
        request = c.ServerPoolService_getAgentVersion()
        request._serverId   = serverId
        try:
            response = self.srv.getAgentVersion(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                        
    def getAllMembers(self, serverPool): 
        """ 
        Return all virtual servers for the server pool.  
        @param serverPool: 
        @type serverPool: ServerPool
        @return: VirtualServer[]
        """
        request = c.ServerPoolService_getAllMembers()
        request._serverPool   = serverPool
        try:
            response = self.srv.getAllMembers(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def getAllServerPools(self):
        """  
        Returns an array of all server pools managed by the system. 
        @return:  ServerPool[]
        """
        request = c.ServerPoolService_getAllServerPools()
        try:
            response = self.srv.getAllServerPools(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)   
         
    def getMasterAgentStatus(self, serverPool): 
        """ 
        Get master agent status for given server pool id.  
        @param serverPool: 
        @type serverPool: ServerPool
        @return: MasterVirtualServerStatus 
        """
        request = c.ServerPoolService_getMasterAgentStatus()
        request._serverPool  = serverPool
        try:
            response = self.srv.getMasterAgentStatus(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def getMasterServer(self, serverPool):
        """ 
        Return the mater server configured for the server pool by pool id. 
        @param serverPool: 
        @type serverPool: ServerPool
        @return: MasterVirtualServer
        """
        request = c.ServerPoolService_getMasterServer()
        request._serverPool = serverPool
        try:
            response = self.srv.getMasterServer(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
          
    def getNetworkBridges(self, serverPool): 
        """ 
        Get xen bridge from a server pool.  
        @param serverPool: 
        @type serverPool: ServerPool
        @return: Map        
        """
        request = c.ServerPoolService_getNetworkBridges()
        request._serverPool = serverPool
        try:
            response = self.srv.getNetworkBridges(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def getNetworkBridgesByServerIP(self, virtualServer): 
        """ 
        Get xen brige by virtualServer ip  
        @param virtualServer: 
        @type virtualServer: VirtualServer
        @return: Map
        """
        request = c.ServerPoolService_getNetworkBridgesByServerIP()
        request._virtualServer = virtualServer
        try:
            response = self.srv.getNetworkBridgesByServerIP(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def getServer(self, vs): 
        """ 
        Return a specific instance of the virtual server by virtualServer id.  
        @param vs: 
        @type vs: VirtualServer
        @return: VirtualServer
        """
        request = c.ServerPoolService_getServer()
        request._virtualServer = virtualServer
        try:
            response = self.srv.getServer(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def getServerByName(self, serverPool, vs): 
        """ 
        Return an instance of the virtual server by pool name and server name from a server pool, by name.  
        ServerPool serverPool, VirtualServer virtualServer
        @param serverPool:
        @type serverPool: ServerPool
        @param vs:
        @type vs: VirtualServer
        @return: VirtualServer  
        """
        request = c.ServerPoolService_getServerByName()
        request._serverPool = serverPool
        request._vs = vs
        try:
            response = self.srv.getServerByName(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getVirtualServerByName(self, serverPoolName, virtualServerName): 
        """ 
        Return an instance of the virtual server by pool name and server name from a server pool, by name.  
        ServerPool serverPool, VirtualServer virtualServer
        @param serverPool:
        @type serverPool: ServerPool
        @param vs:
        @type vs: VirtualServer
        @return: VirtualServer  
        """
        request = c.ServerPoolService_getVirtualServerByName()
        request._serverPoolName = serverPoolName
        request._virtualServerName = virtualServerName
        try:
            response = self.srv.getVirtualServerByName(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    
    def getServerPool(self, serverPool): 
        """ 
        Return instances of server pool by id managed by the system. 
        @param serverPool: 
        @type serverPool: ServerPool
        @return: ServerPool
        """ 
        request = c.ServerPoolService_getServerPool()
        request._serverPool = serverPool
        try:
            response = self.srv.getServerPool(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def getServerPoolById(self, poolId): 
        """ 
        Return instances of server pool by id managed by the system. 
        @param serverPool: 
        @type serverPool: ServerPool
        @return: ServerPool
        """ 
        request = c.ServerPoolService_getServerPoolById()
        request._poolId = poolId
        try:
            response = self.srv.getServerPoolById(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                      
    def getServerPoolByName(self, poolName):
        """ 
        Return instance os a specific server pool managed by the system, by its name.
        @param poolName: 
        @type poolName: str
        @return: ServerPool
        """
        c.ServerPoolService_getServerPoolByName
        request = c.ServerPoolService_getServerPoolByName()
        request._poolName = poolName
        try:
            response = self.srv.getServerPoolByName(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def rebootServer(self, server): 
        """ 
        Reboot the virtual server.  
        @param server:
        @type server: VirtualServer
        @return: void 
        """
        request = c.ServerPoolService_rebootServer()
        request._server = server
        try:
            response = self.srv.rebootServer(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def refreshServerPool(self, pool): 
        """ 
        Refresh server and server pool status , synchronize db data between EM-OVM and Agent db. 
        @param pool: 
        @type pool: ServerPool
        @return: void
        """ 
        request = c.ServerPoolService_refreshServerPool()
        request._pool = pool
        try:
            response = self.srv.refreshServerPool(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                
    def removeMember(self, server, force=False, warningMessageList=None): 
        """ 
        Remove a virtual server from the server pool (logical de-referencing).  
        VirtualServer server
        @param server: 
        @type server: 
        @return: void
        """
        request = c.ServerPoolService_removeMember()
        request._server = server
        request._force = force
        request._warningMessageList = warningMessageList
        try:
            response = self.srv.removeMember(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def restoreVirtualServerPoolByManagerData(self, serverPoolId): 
        """ 
        Restore ServerPool.  
        @param serverPoolId: 
        @type serverPoolId: Long
        @return: void
        """
        request = c.ServerPoolService_restoreVirtualServerPoolByManagerData()
        request._serverPoolId = serverPoolId
        try:
            response = self.srv.restoreVirtualServerPoolByManagerData(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def shutdownServer(self, server): 
        """ 
        Shut down the virtual server. 
        VirtualServer server
        @param server: 
        @type server: VirtualServer
        @return: void 
        """ 
        request = c.ServerPoolService_shutdownServer()
        request._server = server
        try:
            response = self.srv.shutdownServer(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                
    def updateServerPool(self, poolId, HAStatus, newPoolName): 
        """ 
        Update server pool info, including: 
        1.Update name of server pool; 
        2.Enable/Disable HA status for server pool. 
        @param poolId: 
        @type poolId: Long
        @param HAStatus: 
        @type HAStatus: boolean
        @param newPoolName: 
        @type newPoolName: str
        @return: void
        """
        request = c.ServerPoolService_updateServerPool()
        request._poolId      = poolId
        request._HAStatus    = HAStatus
        request._newPoolName = newPoolName
        try:
            response = self.srv.updateServerPool(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def updateServerPoolVIP(self, poolId, HAStatus, newPoolName, serverPoolVirtualIP): 
        """ 
        Update server pool info, including: 
        1.Update name of server pool; 
        2.Enable/Disable HA status for server pool; 
        3.Update serverPoolVirtualIP.
        @param poolId: 
        @type poolId: Long
        @param HAStatus: 
        @type HAStatus: boolean
        @param newPoolName: 
        @type newPoolName: str
        @param serverPoolVirtualIP:
        @type serverPoolVirtualIP: str  
        @return: void
        """
        request = c.ServerPoolService_updateServerPool2()
        request._poolId      = poolId
        request._HAStatus    = HAStatus
        request._newPoolName = newPoolName
        request._serverPoolVirtualIP = serverPoolVirtualIP
        try:
            response = self.srv.updateServerPool2(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                     
    def updateUtilityServerPassword(self, hostName, oldPassword, newPassword): 
        """ 
        Update password for utility virtual server.  
        @param hostName: 
        @type hostName: str
        @param oldPassword: 
        @type oldPassword: str
        @param newPassword: 
        @type newPassword: str
        @return: void
        """
        request = c.ServerPoolService_updateUtilityServerPassword()
        request._hostName       = hostName
        request._oldPassword    = oldPassword
        request._newPassword    = newPassword
        try:
            response = self.srv.updateUtilityServerPassword(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def updateVirtualServer(self, vs, force, warningMessageList = None): 
        """ 
        Update Master and regular Virtual Server information by server id.  
        @return: void
        """
        request = c.ServerPoolService_updateVirtualServerById()
        request._vs       = vs
        request._force    = force
        request._warningMessageList = warningMessageList
        try:
            response = self.srv.updateVirtualServerById(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def updateVirtualServerAgentPassword(self, hostName, oldPassword, newPassword): 
        """ 
        Update virtual server agent's password.  
        @param hostName: 
        @type hostName: str
        @param oldPassword: 
        @type oldPassword: str
        @param newPassword: 
        @type newPassword: str
        @return: void
        """
        request = c.ServerPoolService_updateVirtualServerAgentPassword()
        request._hostName       = hostName
        request._oldPassword    = oldPassword
        request._newPassword    = newPassword
        try:
            response = self.srv.updateVirtualServerAgentPassword(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                
    def validateAddMembers(self, serverPool, serverConfigs): 
        """ 
        Do validation job before add server into one server pool: 
        1.Is there no new server be added in the serverConfigs. 
        2.Check the Integrity for all new server. 
        3.Check all new server's agent can be connected or not. 
        4.Check all new server is managed or not, in another word, the server's management is exclusive. 
        5.If current server pool is HA enabled, all of new server's ha ability will be check. 
        @param serverPool:
        @type serverPool: ServerPool
        @param serverConfigs:
        @type serverConfigs: VirtualServerConfig[]
        @return: void  
        """ 
        request = c.ServerPoolService_validateAddMembers()
        request._serverPool       = serverPool
        request._serverConfigs    = serverConfigs
        try:
            response = self.srv.validateAddMembers(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def validateBlackoutServer(self, virtualServerID): 
        """     
        @return void
        """
        request = c.ServerPoolService_validateBlackoutServer()
        request._virtualServerID          = virtualServerID
        try:
            response = self.srv.validateBlackoutServer(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                            
    def validateCreateServerPool(self, poolConfig): 
        """  
        Do validation job before creation server pool: 
        1.Check the Integrity for the new server pool. 
        2.Check all new server's agent can be connected or not. 
        3.Check all new server is managed or not, in another word, the server's management is exclusive. 
        4.If setcurrent server pool's HA enabled, all of new server's ha ability will be check. 
        @param poolConfig:
        @type poolConfig: ServerPoolConfig
        @return: void 
        """ 
        request = c.ServerPoolService_validateCreateServerPool()
        request._poolConfig       = poolConfig
        try:
            response = self.srv.validateCreateServerPool(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def validateDeleteServerPool(self, serverPool): 
        """ 
        Do validation job before delete a server Pool. 
        @param serverPool:
        @type serverPool: ServerPool 
        @return: void 
        """ 
        request = c.ServerPoolService_validateDeleteServerPool()
        request._serverPool       = serverPool
        try:
            response = self.srv.validateDeleteServerPool(request)
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def validateRebootServer(self, vs): 
        """ 
        Do validation job before reboot server. 
        1.The server's agent and its master agent can be connected or not?  
        @param vs: 
        @type vs: VirtualServer
        @return: void
        """
        request = c.ServerPoolService_validateRebootServer()
        request._vs       = vs
        try:
            response = self.srv.validateRebootServer(request)
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def validateRemoveMember(self, vs): 
        """ 
        Do validation job before remove server from one server pool. 
        1.The server's agent and its master agent can be connected or not. 
        2.Are there any running vm in the server, if there are, the server couldn't be removed. 
        3.If the server is a master server, so the server couldn't be removed directly.  
        @param vs: 
        @type vs: VirtualServer
        @return: void
        """
        request = c.ServerPoolService_validateRemoveMember()
        request._vs       = vs
        try:
            response = self.srv.validateRemoveMember(request)
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                
    def validateShutdownServer(self, vs): 
        """ 
        Do validation jobs before shutdown one server. 
        1.Check the server's agent can be connected or not. 
        2.Check the server's master agent can be connected or not.  
        @param vs: 
        @type vs: VirtualServer
        @return: void
        """
        request = c.ServerPoolService_validateShutdownServer()
        request._vs       = vs
        try:
            response = self.srv.validateShutdownServer(request)
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def validateUpdateUtilityServerPassword(self, hostName, oldPassword): 
        """ 
        Validation user and password for utility server. 
        1.Try to connect utility virtual server using oldpassword. 
        2.If connection failed, then validtion will fail, and throw error exception message. 
        @param hostName: 
        @type hostName: str
        @param oldPassword: 
        @type oldPassword: str
        @return: void       
        """ 
        request = c.ServerPoolService_validateUpdateUtilityServerPassword()
        request._hostName          = hostName
        request._oldPassword       = oldPassword
        try:
            response = self.srv.validateUpdateUtilityServerPassword(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    def validateUpdateVSAgentPassword(self, hostName, oldPassword): 
        """ 
        Validation virtual server agent password can be updated or not.
        1.Try to connect virtual server agent usting oldPassword. 
        2.If failed, then validationg will fail, and throw error exception message.  
        @param hostName: 
        @type hostName: str
        @param oldPassword: 
        @type oldPassword: str
        @return: void  
        """      
        request = c.ServerPoolService_validateUpdateVSAgentPassword()
        request._hostName          = hostName
        request._oldPassword       = oldPassword
        try:
            response = self.srv.validateUpdateVSAgentPassword(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)


    def checkHAAbility4ServerPoolById(self, poolId, silent=True): 
        """ 
        @return: void  
        """      
        request = c.ServerPoolService_checkHAAbility4ServerPoolById()
        request._poolId          = poolId
        try:
            response = self.srv.checkHAAbility4ServerPoolById(request)
            return True
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent) 


    def getVirtualServerMetricsById(self, id, silent = True): 
        """ 
        @return: list 
        """      
        request = c.ServerPoolService_getVirtualServerMetrics()
        request._virtualServerIDList          = [long(id)]
        try:
            response = self.srv.getVirtualServerMetrics(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)             

    def getServerPoolMetricsByServerPoolId(self, serverPoolId, silent = True): 
        """ 
        @return: ServerPoolMetrics 
        """      
        request = c.ServerPoolService_getServerPoolMetricsByServerPoolId()
        request._serverPoolId          = serverPoolId
        try:
            response = self.srv.getServerPoolMetricsByServerPoolId(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)  

    def getClusterRootByServerPoolId(self, serverPoolId): 
        """ 
        @return: map 
        """      
        request = c.ServerPoolService_getClusterRootByServerPoolId()
        request._serverPoolId          = serverPoolId
        try:
            response = self.srv.getClusterRootByServerPoolId(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)   

    def getStorageRepositoriesByServerPoolId(self, serverPoolId): 
        """ 
        @return: map 
        """      
        request = c.ServerPoolService_getStorageRepositoriesByServerPoolId()
        request._serverPoolId          = serverPoolId
        try:
            response = self.srv.getStorageRepositoriesByServerPoolId(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent) 
          

    def updateUserList4ServerPool(self, serverPoolId, userIDList): 
        """ 
        @return: void
        """      
        request = c.ServerPoolService_updateUserList4ServerPool()
        request._serverPoolId          = serverPoolId
        request._userIDList            = userIDList
        try:
            response = self.srv.updateUserList4ServerPool(request)
            return True
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)
                          
    def getMinimumAgentVersion(self, serverPoolIDs): 
        """ 
        @return: void
        """      
        request = c.ServerPoolService_getMinimumAgentVersion()
        request._serverPoolIDs          = serverPoolIDs
        try:
            response = self.srv.getMinimumAgentVersion(request)
            response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)

