#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

#from zsigen import VirtualMachineService_client as c
from ZSI import FaultException
from util import handle_faultexception, handle_exception
import sys
class VirtualMachineManagementPort:

    def __init__(self, urlbase, auth, transport, ovm_version):
        self.wspath = "/VirtualMachineServiceSoapHttpPort"
        global c
        package = "zsigen." + ovm_version
        exec("from " + package + " import VirtualMachineService_client as client")
        c = client
        self.loc = c.VirtualMachineServiceLocator()
        self.srv = self.loc.getVirtualMachineServiceSoapHttpPort(url = urlbase + self.wspath, 
                                                                 auth = auth,                                             
                                                                 #tracefile=sys.stdout, 
                                                                 transport = transport)
    def deleteVM(self, vmId):
        """ 
        Delete a virtual machine. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @return: void 
        """
        request = c.VirtualMachineService_deleteVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.deleteVMByVMId(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
               
    def getAllAlertAssociatedWithImg(self, vm): 
        """ 
        Query all alerts associated with a vm  
        @param vm: 
        @type vm: VirtualMachine
        @return: List
        """
        request = c.VirtualMachineService_getAllAlertAssociatedWithImg()
        request._vm = vm
        try:
            response = self.srv.getAllAlertAssociatedWithImg(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
            
    def getAllOperatingSystem(self): 
        """ 
        Query all available Operating System  
        @return: An Array of OperatingSystem
        """
        request = c.VirtualMachineService_getAllOperatingSystemArray()
        try:
            response = self.srv.getAllOperatingSystemArray(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def getAllVMs(self, serverPool):
        """  
        get vms by serverPoolID 
        @param serverPool: 
        @type serverPool: ServerPool
        @return:  VirtualMachine[]
        """
        
        request = c.VirtualMachineService_getAllVMs2()
        request._serverPool = serverPool
        try:
            response = self.srv.getAllVMs2(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                
    def getAllVMs_By_Server(self, virtualServer):
        """  
        get vms by serverID
        @param virtualServer: 
        @type virtualServer: VirtualServer
        @return:  VirtualMachine[]
        """
        request = c.VirtualMachineService_getAllVMs()
        request._virtualServer = virtualServer
        try:
            response = self.srv.getAllVMs(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def getAssociatedPool(self, virtualMachine): 
        """ 
        Return server pool details, where the virtual machine is physically (and logically) stored.  
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @return:  ServerPool
        """
        request = c.VirtualMachineService_getAssociatedPool()
        request._virtualMachine = virtualMachine
        try:
            response = self.srv.getAssociatedPool(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def getAssociatedServer(self, virtualMachine, silent=True): 
        """ 
        Return virtual server details, currently hosting the virtual machine. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @return:  VirtualServer        
        """ 
        request = c.VirtualMachineService_getAssociatedServer()
        request._virtualMachine = virtualMachine
        try:
            response = self.srv.getAssociatedServer(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e, silent)
            pass
        except Exception, e:
            handle_exception(e, silent)
    
    def getLastAlertAssociatedWithImg(self, vm): 
        """ 
        Query latest alert associated with a vm  
        @param vm: 
        @type vm: VirtualMachine
        @return: AlertLog
        """
        request = c.VirtualMachineService_getLastAlertAssociatedWithImg()
        request._vm = vm
        try:
            response = self.srv.getLastAlertAssociatedWithImg(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
    
    def getRealPath(self, vm): 
        """ 
        Get the absolute path of a virtual machine in the file system.  
        VirtualMachine vm
        @param vm: 
        @type vm: VirtualMachine
        @return: str 
        """
        request = c.VirtualMachineService_getRealPath()
        request._vm = vm
        try:
            response = self.srv.getRealPath(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    
    def getVM(self, virtualMachine):
        """ 
        Return an instance of the virtual machine details. 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @return: VirtualMachine
        """
        request = c.VirtualMachineService_getVM()
        request._virtualMachine = virtualMachine
        try:
            response = self.srv.getVM(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
          
    def getVMByName(self, serverPoolName, name, silent = False):
        """ 
        Return an instance of the virtual machine details, by its name 
        @param serverPool: 
        @type serverPool: ServerPool
        @param name: 
        @type name: str
        @return: VirtualMachine       
        """
        request = c.VirtualMachineService_getVMByName()
        request._serverPoolName = serverPoolName
        request._name       = name
        try:
            response = self.srv.getVMByName(request)
            return response._result 
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
          
    def getVncPassword(self, vm): 
        """ 
        Get vnc password  
        @param vm: 
        @type vm: VirtualMachine
        @return: str
        """
        request = c.VirtualMachineService_getVncPassword()
        request._vm       = vm
        try:
            response = self.srv.getVncPassword(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
         
    
    def pauseVM(self, vmId): 
        """ 
        pause vm 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine 
        @return: void
        """
        request = c.VirtualMachineService_pauseVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.pauseVMByVMId(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
         
    def powerOffVM(self, vmId): 
        """ 
        Power off a virtual machine  
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine 
        @return: void
        """
        request = c.VirtualMachineService_powerOffVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.powerOffVMByVMId(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
         
    def powerOnVM(self, vmId):
        """ 
        Power on a virtual machine 
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine 
        @return: void
        """
        request = c.VirtualMachineService_powerOnVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.powerOnVMByVMId(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
         
    def queryLocalDiskInfo(self, disk): 
        """ 
        Query virtual disk status 
        @param disk: 
        @type disk: VirtualDisk 
        @return:  VirtualDisk
        """
        request = c.VirtualMachineService_queryLocalDiskInfo()
        request._disk = disk
        try:
            response = self.srv.queryLocalDiskInfo(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
           
    def queryLocalDisks(self, vm): 
        """ 
        Query local disks of a vm 
        @param vm: 
        @type vm: VirtualMachine
        @return: List
        """
        request = c.VirtualMachineService_queryLocalDisks()
        request._vm = vm
        try:
            response = self.srv.queryLocalDisks(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
          
    def queryCdrom(self, vm): 
        """ 
        Query cdroms of a vm 
        @param vm: 
        @type vm: VirtualMachine
        @return: CdConfig[]
        """
        request = c.VirtualMachineService_queryCdromArray()
        request._vm = vm
        try:
            response = self.srv.queryCdromArray(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)        
    
    def queryNetworkInterfaceCards(self, vm) :
        """ 
        Query network interface cards of a vm  
        @param vm: 
        @type vm: VirtualMachine
        @return: VirtualNetworkCard[]
        """
        request = c.VirtualMachineService_queryNetworkInterfaceCardArray()
        request._vm = vm
        try:
            response = self.srv.queryNetworkInterfaceCardArray(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    
    def queryPreferredServer(self, vm): 
        """ 
        Query preferred servers of a vm  
        @param vm: 
        @type vm: VirtualMachine
        @return: List
        """
        request = c.VirtualMachineService_queryPreferredServer()
        request._vm = vm
        try:
            response = self.srv.queryPreferredServer(request)
            return response._result._item
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def queryPreferredServerArray(self, vmId): 
        """ 
        VirtualServer[] 
        """
        request = c.VirtualMachineService_queryPreferredServerArray()
        request._vmId = vmId
        try:
            response = self.srv.queryPreferredServerArray(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)        
    
    def querySharedDiskInfo(self, vm, disk):
        """
        @param vm: 
        @type vm: VirtualMachine
        @param disk: 
        @type disk: VirtualDisk
        @return: VirtualDisk
        """
        request = c.VirtualMachineService_querySharedDiskInfo()
        request._vm   = vm
        request._disk = disk
        try:
            response = self.srv.querySharedDiskInfo(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
                
    def querySharedVirtualDiskArray(self, vmId): 
        """ 
        Query shared disks of a vm 
        @param vmId: 
        @type vmId: VirtualMachine
        @return: SharedVirtualDisk[]
        """
        request = c.VirtualMachineService_querySharedVirtualDiskArray()
        request._vmId   = vmId
        try:
            response = self.srv.querySharedVirtualDiskArray(request)
            return response._result
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    
    def queryVMStatus(self, virtualMachine): 
        """ 
        Query virtual machine status  
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine 
        @return: VirtualMachineBase.VmStatus       
        """
        request = c.VirtualMachineService_queryVMStatus()
        request._virtualMachine   = virtualMachine
        try:
            response = self.srv.queryVMStatus(request)
        except Exception, e:
            print e
            raise
        return response._result
    
    def rebootVM(self, vmId): 
        """ 
        Reboot a virtual machine  
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @return: void        
        """
        request = c.VirtualMachineService_rebootVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.rebootVMByVMId(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def resumeVM(self, vmId): 
        """ 
        Resume a suspended virtual machine.   
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine 
        @return: void      
        """
        request = c.VirtualMachineService_resumeVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.resumeVMByVMId(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                
    def saveAsTemplate(self, vm, templateName): 
        """ 
        save as template --must set imgId of parameter object  
        @param vm: 
        @type vm: VirtualMachine
        @param templateName: 
        @type templateName: str
        @return: void
        """
        request = c.VirtualMachineService_saveAsTemplateAsync()
        request._vm   = vm
        request._templateName   = templateName
        try:
            response = self.srv.saveAsTemplateAsync(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)                        

    def setVMName(self, virtualMachine, val): 
        """ 
        Sets the virtual machine name.  
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine
        @param val: 
        @type val: str
        @return: void
        """
        request = c.VirtualMachineService_setVMName()
        request._virtualMachine   = virtualMachine
        request._val              = val
        try:
            response = self.srv.setVMName(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def suspendVM(self, vmId): 
        """ 
        Suspend a running virtual machine.  
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine 
        @return: void       
        """
        request = c.VirtualMachineService_suspendVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.suspendVMByVMId(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def unpauseVM(self, vmId): 
        """ 
        unpause vm  
        @param virtualMachine: 
        @type virtualMachine: VirtualMachine        
        @return: void
        """
        request = c.VirtualMachineService_unpauseVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.unpauseVMByVMId(request)
            return True
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
        
    def update(self, object): 
        """ 
        Update information  
        @param object: 
        @type object: Object
        @return: Object
        """
        request = c.VirtualMachineService_update()
        request._object   = object
        try:
            response = self.srv.update(request)
            return response._result  
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def unregisterVM(self, vmId):
        """ return void """ 
        request = c.VirtualMachineService_unregisterVMByVMId()
        request._vmId = vmId
        try:
            response = self.srv.unregisterVMByVMId(request)
            return True  
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)

    def queryVMStatusByVMId(self, vmId):
        """ return VmStatus """ 
        request = c.VirtualMachineService_queryVMStatusByVMId()
        request._vmId = vmId
        try:
            response = self.srv.queryVMStatusByVMId(request)
            return response._result  
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)
                         

    def queryVMMetricByVMId(self, vmId):
        """ return VmMetrics  """ 
        request = c.VirtualMachineService_queryVMMetricByVMId()
        request._vmId = vmId
        try:
            response = self.srv.queryVMMetricByVMId(request)
            return response._result  
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)        

    def queryVMMetricArrayByVMIds(self, vmIds):
        """ return VmMetrics  """ 
        request = c.VirtualMachineService_queryVMMetricArrayByVMIds()
        request._vmIds = vmIds
        try:
            response = self.srv.queryVMMetricArrayByVMIds(request)
            return response._result  
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e) 
             
    def queryVMStatusArrayByVMIds(self, vmIds):
        """ return VmStatus[]  """ 
        request = c.VirtualMachineService_queryVMStatusArrayByVMIds()
        request._vmIds = vmIds
        try:
            response = self.srv.queryVMStatusArrayByVMIds(request)
            return response._result  
        except FaultException, e:
            handle_faultexception(e)
            pass
        except Exception, e:
            handle_exception(e)  
            
    
