#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

import gettext
_ = gettext.gettext

# Sub commands data structure

cmd_list = {
            'vm': [
                    {'subcmd': 'bootdev', 'desc': _('Set primary boot device')},
                    {'subcmd': 'cd', 'desc': _('Virtual machine CD-ROM management'), 'is_subgroup': 1},
                    {'subcmd': 'cd attach', 'desc': _('Attach CD-ROM to a virtual machine'), 'parent':'cd'},
                    {'subcmd': 'cd detach', 'desc': _('Detach CD-ROM from a virtual machine'), 'parent':'cd'},
                    {'subcmd': 'cd ls', 'desc': _('List a virtual machine\'s CD-ROMs'), 'parent':'cd'},
                    {'subcmd': 'clone', 'desc': _('Clone a virtual machine')},
                    {'subcmd': 'conf', 'desc': _('Configure a virtual machine')},
                    {'subcmd': 'del', 'desc': _('Delete a virtual machine')},
                    {'subcmd': 'deploy', 'desc': _('Deploy a virtual machine')},
                    {'subcmd': 'disk', 'desc': _('Virtual machine disk management'), 'is_subgroup': 1},
                    {'subcmd': 'disk add', 'desc': _('Create and add disk to a virtual machine'), 'parent':'disk'},
                    {'subcmd': 'disk conf', 'desc': _('Configure a disk in a virtual machine'), 'parent':'disk'},                  
                    {'subcmd': 'disk del', 'desc': _('Remove disk from a virtual machine'), 'parent':'disk'},
                    {'subcmd': 'disk ls', 'desc': _('List a virtual machine\'s disks'), 'parent':'disk'},
                    {'subcmd': 'info', 'desc': _('Get virtual machine information')},
                    {'subcmd': 'kb', 'desc': _('Set keyboard layout')},
                    {'subcmd': 'ls', 'desc': _('List virtual machines')},
                    {'subcmd': 'mig', 'desc': _('Live migrate a virtual machine')},
                    {'subcmd': 'mig all', 'desc': _('Migrate all virtual machines on a server')},
                    {'subcmd': 'new', 'desc': _('Create a new virtual machine')},
                    {'subcmd': 'nic', 'desc': _('Virtual machine NIC management'), 'is_subgroup': 1},
                    {'subcmd': 'nic add', 'desc': _('Create and add NIC to a virtual machine'), 'parent':'nic'},
                    {'subcmd': 'nic conf', 'desc': _('Configure a NIC in a virtual machine'), 'parent':'nic'},            
                    {'subcmd': 'nic del', 'desc': _('Remove NIC from a virtual machine'), 'parent':'nic'},
                    {'subcmd': 'nic ls', 'desc': _('List a virtual machine\'s NICs'), 'parent':'nic'},
                    {'subcmd': 'pause', 'desc': _('Pause a virtual machine')},
                    {'subcmd': 'poweroff', 'desc': _('Power off a virtual machine')},
                    {'subcmd': 'poweron', 'desc': _('Power on a virtual machine')},
                    {'subcmd': 'reboot', 'desc': _('Restart a virtual machine')},
                    {'subcmd': 'resume', 'desc': _('Resume a virtual machine')},
                    {'subcmd': 'save', 'desc': _('Save virtual machine as a virtual machine template')},
                    {'subcmd': 'sd', 'desc': _('Shared virtual disk management'), 'is_subgroup': 1},
                    {'subcmd': 'sd attach', 'desc': _('Attach a shared virtual disk to a virtual machine'), 'parent':'sd'},
                    {'subcmd': 'sd detach', 'desc': _('Detach a shared virtual disk from a virtual machine'), 'parent':'sd'},
                    {'subcmd': 'stat', 'desc': _('Get/Reset a virtual machine status')},
                    {'subcmd': 'suspend', 'desc': _('Suspend a virtual machine')},
                    {'subcmd': 'unpause', 'desc': _('Unpause a virtual machine')},
                    {'subcmd': 'vncpwd', 'desc': _('Set VNC console password')},
                   ],
            'svr': [
                    {'subcmd': 'add', 'desc': _('Add server to a server pool')},
                    {'subcmd': 'agent', 'desc': _('Get a server\'s agent version')},
                    {'subcmd': 'conf', 'desc': _('Configure a server')},
                    {'subcmd': 'del', 'desc': _('Delete server from a server pool')},
                    {'subcmd': 'info', 'desc': _('Get server information')},
                    {'subcmd': 'ls', 'desc': _('List servers')},
                    {'subcmd': 'mode', 'desc': _('Set server mode')},
                    {'subcmd': 'poweroff', 'desc': _('Power off a server')},
                    {'subcmd': 'pwd', 'desc': _('Change utility server or agent password')},
                    {'subcmd': 'restart', 'desc': _('Reboot a server')},
                    {'subcmd': 'stat', 'desc': _('Get server status')},
                   ],
            'svrp':[
                    {'subcmd': 'conf', 'desc': _('Configure a server pool')},
                    {'subcmd': 'del', 'desc': _('Delete a server pool')},
                    {'subcmd': 'info', 'desc': _('Get server pool information')},
                    {'subcmd': 'ls', 'desc': _('List server pools')},
                    {'subcmd': 'new', 'desc': _('Create a new server pool')},
                    {'subcmd': 'refresh', 'desc': _('Refresh all server pools')},
                    {'subcmd': 'restore', 'desc': _('Restore server pool information')},
                    {'subcmd': 'stat', 'desc': _('Get server pool status')},                  
                  ],
            'img':[
                    {'subcmd': 'approve', 'desc': _('Approve a virtual machine image')},
                    {'subcmd': 'del', 'desc': _('Delete a virtual machine image')},
                    {'subcmd': 'disc', 'desc': _('List all discoverable virtual machine images')},
                    {'subcmd': 'imp', 'desc': _('Import virtual machine image from an external source')},
                    {'subcmd': 'ls', 'desc': _('List virtual machine images')},
                    {'subcmd': 'reg', 'desc': _('Register a discoverable virtual machine image')},
                    {'subcmd': 'stat', 'desc': _('Get virtual machine image status')},
                   ],
            'iso':[
                    {'subcmd': 'approve', 'desc': _('Approve an ISO image')},              
                    {'subcmd': 'del', 'desc': _('Delete an ISO image')},
                    {'subcmd': 'disc', 'desc': _('List all discoverable ISO images')},
                    {'subcmd': 'imp', 'desc': _('Import ISO image from an external source')},                  
                    {'subcmd': 'ls', 'desc': _('List ISO images')},
                    {'subcmd': 'reg', 'desc': _('Register a discoverable ISO image')},
                    {'subcmd': 'stat', 'desc': _('Get ISO image status')},                   
                   ],
            'tmpl':[
                    {'subcmd': 'approve', 'desc': _('Approve a virtual machine template')},
                    {'subcmd': 'del', 'desc': _('Delete a virtual machine template')},
                    {'subcmd': 'disc', 'desc': _('List all discoverable virtual machine templates')},
                    {'subcmd': 'imp', 'desc': _('Import virtual machine template from an external source')},
                    {'subcmd': 'ls', 'desc': _('List virtual machine templates')},
                    {'subcmd': 'reg', 'desc': _('Register a discoverable virtual machine template')},
                    {'subcmd': 'stat', 'desc': _('Get virtual machine template status')},                   
                   ],
            'sd':[
                    {'subcmd': 'del', 'desc': _('Delete a shared virtual disk')},
                    {'subcmd': 'ls', 'desc': _('List shared virtual disks')},
                    {'subcmd': 'new', 'desc': _('Create and register a shared virtual disk')},
                    {'subcmd': 'stat', 'desc': _('Get shared virtual disk status')},
                    {'subcmd': 'disc', 'desc': _('List all discoverable shared virtual disks')},
                    {'subcmd': 'reg', 'desc': _('Register a discoverable shared virtual disk')},
                  ],
            'user':[
                    {'subcmd': 'group', 'desc': _('Group assignment/removal'), 'is_subgroup': 1},  
                    {'subcmd': 'group add', 'desc': _('Assign user to a group'), 'parent':'group'},
                    {'subcmd': 'ls', 'desc': _('List all users')},
                    {'subcmd': 'new', 'desc': _('Create a new user account')},
                    {'subcmd': 'svrp', 'desc': _('Server pool assignment/removal'), 'is_subgroup': 1},
                    {'subcmd': 'svrp add', 'desc': _('Assign user to a server pool'), 'parent':'svrp'},
                    {'subcmd': 'svrp del', 'desc': _('Remove user from a server pool'), 'parent':'svrp'},

                    ],
            'group':[
                    {'subcmd': 'ls', 'desc': _('List all groups')},
                    {'subcmd': 'new', 'desc': _('Create a new group')},                     
                    ], 
            'val':[
                    {'subcmd': 'bootdev', 'desc': _('List all available boot devices')},
                    {'subcmd': 'kb', 'desc': _('List all available keyboard layouts')},
                    {'subcmd': 'os', 'desc': _('List all available operating system types')},
                    
                    
                  ],        
            'use':[],        
            'connect':[],
            'config':[],    
            'shell':[],
            'vncviewer':[],
            'home':[],
            'help':[],
            }

group_info = {
            'vm': _('Virtual machine management'),
            'svr': _('Server management'),
            'svrp': _('Server pool management'),
            'img': _('Virtual machine image management'),
            'iso': _('ISO image management'),
            'tmpl': _('Virtual machine template management'),
            'sd': _('Shared virtual disk management'),
            'user': _('User management'),
            'group': _('Group management'),
            'val': _('System parameter management'),      
            }

standalone_cmd_info = {
            'use': _('Specify a server pool to use'),       
            'connect': _('Connect to Oracle VM Manager from CLI shell'),
            'config': _('CLI configuration'),  
            'shell': _('Launch an interactive shell'),
            'vncviewer':_('Start a VNC console'),
            'help': _('Show help'),
            'exit': _('Exit'),            
            }
           
class OvmCmd:
    def __init(self):
        self.cmd_list = cmd_list
    def get_all_groups(self):
        groups = [g for g in group_info]
        groups.sort()
        return groups
    
    def is_group(self, group_name):
        result = [g for g in cmd_list if g == group_name]
        if [] == result:
            return False
        else:
            return True
        
    def get_group_by_name(self, group_name):
        for g in cmd_list: 
            if g == group_name:
                return g
        return None

    def get_subcmds_by_group(self, group_name):
        for g in cmd_list: 
            if g == group_name:
                return [s['subcmd'] for s in cmd_list[g]]
        return None

    def get_all_subcmds(self):
        result = []
        for g in cmd_list: 
            for s in cmd_list[g]:
                result.append(g + ' ' + s['subcmd'])
        return result
    
    def get_subcmd_by_group(self, group_name, cmd_name):
        for g in cmd_list: 
            if g == group_name:
                try:
                    return [s['subcmd'] for s in cmd_list[g] if s['subcmd']== cmd_name][0]
                except:
                    pass
        return None
      
    def get_group_infos(self):
        return group_info

    def get_group_desc_by_name(self, group_name):
        try:
            return group_info[group_name]
        except:
            pass
    
    def get_standalone_cmd_infos(self):
        return standalone_cmd_info

    def get_standalone_cmds(self):
        return [s for s in standalone_cmd_info]
                    
    def get_cmd_list(self):
        return cmd_list                    

    def get_subgroups_by_group(self, group_name):
        return [s['subcmd'] for s in cmd_list[group_name] if s.has_key('is_subgroup')]

    def get_all_subgroup_cmds(self):
        result = []
        for g in cmd_list:
            for s in cmd_list[g]:
                if s.has_key('is_subgroup'):
                    result.append((g+"_" + s['subcmd']).replace(' ', '_'))
        return result

        
    def is_subgroup(self, group_name, subgroup_name):
        if subgroup_name in self.get_subgroups_by_group(group_name):
            return True
        return False

    def get_subsubcmds_by_group(self, group_name, subgroup_name):
        return [s['subcmd'].replace(subgroup_name + ' ', '') for s in cmd_list[group_name] if s.has_key('parent') and s['parent'] == subgroup_name]
    
    def get_cmd_desc(self, cmd):
        cmds = cmd.split('_')
        if len(cmds) == 1:
            try:
                return [standalone_cmd_info[s] for s in standalone_cmd_info if s == cmds[0]][0]
            except:
                pass
        else:
            group = cmds[0]
            subcmd = " ".join(cmds[1:])
            return [s['desc'] for s in cmd_list[group] if s['subcmd']== subcmd][0]




                