#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

import os, sys, gettext
import ConfigParser

_ = gettext.gettext

MESSAGES = {'hostname':    [_("Oracle VM Manager hostname: "), False, False],
            'port':        [_("Oracle VM Manager port number: "), False, False],
            'deploy_path': [_("Deploy path (leave blank for default): "), True, False],
            'vncviewer':   [_("Location of vncviewer (leave blank to skip): "), True, True],
            'https': [_("Enable HTTPS support? (Y/n): "), True, False]            
            }

class OvmConfigFile:
    def __init__(self, init=True):
        ovm_home = get_ovm_home()
        self.configfile = os.path.join(ovm_home, 'ovm.conf')
        self.configparser = ConfigParser.ConfigParser()
        if init:
            if not os.path.exists(self.configfile):
                self.config()
    
    def getconfig(self, param_name):
        self.configparser.read(self.configfile)
        try:
            param_value = self.configparser.get("ovm", param_name)
            return param_value
        except Exception:
            print _("You don't have \"%(name)s\" configured.") %{"name":param_name}
            self.setconfig(param_name)  


    def get(self, param_name):
        self.configparser.read(self.configfile)
        param_value = self.configparser.get("ovm", param_name)
        return param_value
            
    def setconfig(self, param_name):
        self.configparser.read(self.configfile)
        try:
            value = self.setvalue(MESSAGES[param_name][0], MESSAGES[param_name][1], MESSAGES[param_name][2])            
            if param_name == 'https':
                if value.lower() == 'y' or value =="":
                    value = 'enabled'
                else:
                    value = 'disabled'
            if param_name == "vncviewer":
                if value == "":
                    return                   
            self.configparser.set("ovm", param_name, value)
            self.configparser.write(open(self.configfile, "w"))
            print _("\nConfiguration complete.")
            print _("Please rerun the Oracle VM Manager Command Line Interface.")
            sys.exit()
        except SystemExit:
            sys.exit()    
        except Exception:
            print _("Error occurred while setting param \"%(name)s\"") % {"name":param_name}
            sys.exit() 
                        
    def setvalue(self, message, allow_empty = False, check_file = False):
        param = ""
        while len(param) == 0:
            try:
                param = raw_input(message)
                if len(param) == 0 and allow_empty:
                    break
                if check_file and len(param) != 0:
                    if not os.path.exists(param):
                        print _("File \"%(name)s\" does not exist") % {"name":param}
                        param = ""
            except EOFError:
                sys.exit()
                
        return param

    def set(self, param, value):
        self.configparser.read(self.configfile)
        if not self.configparser.has_section('ovm'):
            self.configparser.add_section('ovm')
        self.configparser.set("ovm", param, value)          
        self.configparser.write(open(self.configfile, "w"))
                    
    def config(self):
        print _("This is a wizard for configuring the Oracle VM Manager Command Line Interface. ")
        print _("CTRL-C to exit.\n")

        hostname = self.setvalue(_("Oracle VM Manager hostname: "))
        port = self.setvalue(_("Oracle VM Manager port number: "))
        deploy_path = self.setvalue(_("Deploy path (leave blank for default): "), allow_empty = True)
        vncviewer = self.setvalue(_("Location of vncviewer (leave blank to skip): "), allow_empty = True, check_file = True)
        https = self.setvalue(_("Enable HTTPS support? (Y/n): "), allow_empty = True)
        if https == "":
            https = 'y'
        if https.lower() == 'y':
            https = 'enabled'
        else:
            https = 'disabled'
        
        self.configparser.read(self.configfile)
        if not self.configparser.has_section('ovm'):
            self.configparser.add_section('ovm')
        self.configparser.set("ovm", "hostname", hostname)
        self.configparser.set("ovm", "port", port)
        self.configparser.set("ovm", "deploy_path", deploy_path)
        self.configparser.set("ovm", "vncviewer", vncviewer)
        self.configparser.set("ovm", "https", https)
            
        self.configparser.write(open(self.configfile, "w"))
        
        print _("\nConfiguration complete.")
        print _("Please rerun the Oracle VM Manager Command Line Interface.")
        sys.exit(0)
            
def get_ovm_home():
    if sys.platform.startswith('win'):
        if os.environ.has_key('HOME'):
            _path = os.environ['HOME']
        else:
            _path = os.getenv("USERPROFILE")
    else:
        _path = os.environ['HOME']
    ovm_home = os.path.join(_path, '.ovm')
    if not os.path.exists(ovm_home):
         os.makedirs(ovm_home)
    return ovm_home