#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

import sys, os
import gettext
from optparse import OptionParser
import constants as CONSTANTS
from Ovm import Ovm
from util import getpass, get_ovm_home, getusername

_ = gettext.gettext

class MainOptionParser(OptionParser):
    def _get_encoding(self, file):
        encoding = getattr(file, "encoding", None)
        if not encoding:
            encoding = sys.getdefaultencoding()
        return encoding
    
    def print_help(self, file=None):
        if file is None:
            file = sys.stdout
        encoding = self._get_encoding(file)
        file.write(unicode(self.format_help(), "UTF-8").encode(encoding, "replace"))     

def get_opt_index(list):
    for op in list:
        if op.startswith('-'):
            return list.index(op)
    return None

def get_sub_cmd(args):
    index = get_opt_index(args)
    if not index:
        subcmd = ' '.join(args[0:])
    else:
        subcmd = ' '.join(args[0:index])
    return subcmd
       
def main(args):
    """This does all the real work""" 
    gettext.bindtextdomain("ovmcli")
    gettext.textdomain("ovmcli")      
    parser = MainOptionParser(usage=_("%prog [options] subcommand [suboptions]"))
    parser.add_option("-u", "--username", help=_("Oracle VM Manager username"))
    parser.add_option("-p", "--password", help=_("Oracle VM Manager password"))
    parser.add_option("-S", "--sep", default = ' ', help=_("Field delimiter for list outputs"))
    parser.disable_interspersed_args()
    options, args = parser.parse_args(args)

    sep = options.sep
    username = options.username
    password = options.password
    bypass = False
    if len(args) == 0:
        args.append('help')
    subcmd = get_sub_cmd(args)
    
    if subcmd in CONSTANTS.NOLOG_CMDS or args[0] == 'help':
        bypass = True
    if '--help' in args or '-h' in args:
        bypass = True
    
#    if subcmd == 'shell' and ('-L' in args or '--nolog' in args):
#        bypass = True
    if subcmd in CONSTANTS.SHELL_ONLY:
        bypass = True
            
    if not bypass:
        ovm_home = get_ovm_home()
        nolog = False
        if subcmd == 'shell' and ('-L' in args or '--nolog' in args):
            nolog = True
        if os.path.exists(os.path.join(ovm_home, 'ovm.conf')) and not nolog:
            #bypass = True    
            if username is None:
                username = getusername()
                if username == CONSTANTS.EOF:
                    return
            if password is None:
                password = getpass(_("Password: "))

    ovm = Ovm(username = username,
                  password = password, 
                  logmode = CONSTANTS.LOG_SHOW, 
                  bypass = bypass, sep = sep)  
    return ovm.ovmcli.runcmd(args)
             
if __name__ == '__main__':
    main(sys.argv[1:])