#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

import re, sys, os, gettext
import socket
import random
import constants as CONSTANTS
from ovmlogging import ovmlogging
import cStringIO,operator


_ = gettext.gettext

def handle_exception(msg, silent=False):
    log = ovmlogging(mode = CONSTANTS.LOG_FILE_SHOW)

    if str(msg) == 'unknown protocol':
        print "ERROR:" + str(msg)
        print _("Please check the configuration.")
        sys.exit()
    if str(msg) == 'HTTP Digest Authorization Failed':
        log.log(_("Access denied. Please check username or password."))
        if not sys.stdin.isatty():
            sys.exit()
        return       
    nows = re.compile("SOAP must understand error.*")
    if nows.search(str(msg)):
        log.log(_("WS-Security is disabled on the server. Please check the configuration."))
        sys.exit()   

    server_error = re.compile("Response is \"text/html\", not \"text/xml\"")
    if server_error.search(str(msg)):
        log.log(_("Access denied or server error. Please check the OVM Manager configuration."))
        sys.exit()
                
    ovsexcept = re.compile(".*oracle\.ovs.*Exception:([\S\s]*)\).*")
    match     =   ovsexcept.search(str(msg))
    if silent:
        return
    if match is None:
        log.log(str(msg))
        sys.exit()
    else:
        log.log(match.group(1))
        
def handle_faultexception(msg, silent = False):
    faultstring = msg.fault.string
    log = ovmlogging(mode = CONSTANTS.LOG_FILE_SHOW)
    
    ws = re.compile(".*wsse:Security.*")
    if ws.search(faultstring):
        log.log(_("WS-Security is enabled on the server. Please check the configuration."))
        sys.exit()
               
    #ovsexcept = re.compile(".*oracle\.ovs.*Exception:\s*([\S\s]*)\).*")
    ovsexcept = re.compile("Internal Server Error \(([\S\s]*)\).*")
    match     =   ovsexcept.search(faultstring)  
    regwsdl   = re.compile(".*unexpected element name.*")
    wsdlmatch = regwsdl.search(faultstring)
    if wsdlmatch is not None:      
        print _("Updating...")
        gen_wsdl()
        print _("Oracle VM Manager CLI has been updated. Please rerun the command.")
        sys.exit(1)
    
    agent = re.compile(".*OVM-1004 XML-RPC Client.*")
    agentmatch = agent.search(faultstring)
    if agentmatch is not None:
        print _("This command is incompatible with this server pool or an error occurred.")
        return
    if silent == False:
        #print msg
        if match is None:
            log.log(msg)
        else:
            log.log(match.group(1))

def gen_wsdl():
    """ Regenerate WSDL stubs"""
    from ZSI.generate import commands
    from ovmconfigfile import OvmConfigFile
    target_dir = os.path.dirname(__file__) + "/zsigen"
    ConfigFile = OvmConfigFile()    
    hostname = ConfigFile.getconfig("hostname")    
    port = ConfigFile.getconfig("port")    
    deploy_path = ConfigFile.getconfig("deploy_path")       
    if deploy_path is None or len(deploy_path) == 0:
        deploy_path = CONSTANTS.DEFAULT_DEPLOY_PATH
    urlbase = "http://%s:%s/%s" % (hostname, port, deploy_path)
    ports = ['LifecycleServiceSoapHttpPort', 
             'ResourceServiceSoapHttpPort',
             'ServerPoolServiceSoapHttpPort',
             'VirtualMachineServiceSoapHttpPort',
             'AdminServiceSoapHttpPort']
    for port in ports:
        wsdlurl = urlbase + "/" + port + "?WSDL"
        args = ['-b','-o'+ target_dir, wsdlurl]
        commands.wsdl2py(args)

def getpass(prompt):
    from getpass import getpass
    
    try:
        if sys.stdin.isatty():
            password = getpass(prompt)
        else:
            password = sys.stdin.readline().rstrip()
        return password
    except EOFError:
        return None


def getusername():
    try:
       username = raw_input(_("Login: "))
       return username 
    except EOFError:
        return CONSTANTS.EOF
       
def get_ovm_home():
    if sys.platform.startswith('win'):
        if os.environ.has_key('HOME'):
            _path = os.environ['HOME']
        else:
            _path = os.getenv("USERPROFILE")
    else:
        _path = os.environ['HOME']
    ovm_home = os.path.join(_path, '.ovm')
    if not os.path.exists(_path):
         os.makedirs(ovm_home)
    return ovm_home

def uniq(list):
   result = []
   for item in list:
     if item not in result:
        result.append(item)
   return result

def indent(rows, sep=" "):
    rows = [[row] for row in rows]
    columns = map(None,*reduce(operator.add,rows))
    max_lengths = [max([len(str(item)) for item in column]) for column in columns]
    output=cStringIO.StringIO()
    for org_rows in rows:
        for row in org_rows:
            if sep == " ":
                print >> output, \
                    sep.join([str.ljust(str(item),width) for (item,width) in zip(row,max_lengths)])
            else:
                print >> output, \
                    sep.join([str.ljust(str(item),0) for (item,width) in zip(row,max_lengths)])                         
    return output.getvalue().rstrip('\n')

   